/*--------------------------------------------------------------------
  This file is part of the NeoMaple library.

  NeoMaple is free software: you can redistribute it and/or modify
  it under the terms of the GNU Lesser General Public License as
  published by the Free Software Foundation, either version 3 of
  the License, or (at your option) any later version.

  NeoMaple is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with NeoMaple.  If not, see
  <http://www.gnu.org/licenses/>.
  --------------------------------------------------------------------*/

#ifndef NEOMAPLE_H
#define NEOMAPLE_H

#if (ARDUINO >= 100)
#include <Arduino.h>
#else
#include <WProgram.h>
#include <pins_arduino.h>
#endif

// 'type' flags for LED pixels (third parameter to constructor):
#define NEO_RGB 0x00 // Wired for RGB data order
#define NEO_GRB 0x01 // Wired for GRB data order
#define NEO_BRG 0x04
#define NEO_RBG 0x08

#define NEO_COLMASK 0x01
#define NEO_KHZ800 0x02 // 800 KHz datastream
#define NEO_SPDMASK 0x02

class MyNeo
{

public:
  // Constructor: number of LEDs, pin number, LED type
  MyNeo();
  ~MyNeo();

  void show(void),
      hide(void),
      setPixelColor(uint16_t n, uint8_t r, uint8_t g, uint8_t b),
      clear();
  uint8_t GetTC();

private:
  uint16_t numBytes;                                    // Size of 'pixels' buffer below
  uint16_t pixels[40 * 24];                              // Holds LED color values (3 bytes each)
  const uint8_t outPins[5] = {PB3, PB4, PB5, PB6, PB7}; //5 pinu funguje PB3-PB7 //upravuj pouze poradi
  void CopyOneBit(uint8_t *fromByte, uint8_t *toByte, uint8_t fromBitNumber, uint8_t toBitNumber);
};

#endif // NeoMaple_H
