#include <Arduino.h>
#include <myAdc.h>
#include <myNeo.h>
#include <libmaple/gpio.h>

//program
int row = 0;
int values[10][20];
int relative[10][20];
int tlacitka[10][20];
int rozdil[10][20];
int mereni = 0;
int ciltivost = 300;
#define pocetmereni 5
int kalibruj = 0;
#define pocetkalibraci 10

//debug
int sloupce = 0;
int radky = 0;

//mulitiplexer
int mux[20] = {0, 1, 2, 3, 4, 5, 6, 8, 9, 10, 11, 12, 13, 14, 16, 17, 18, 19, 20, 21};
int group = 0;

//malovani
#define pocetradku 10
#define pocetsloupcu 20
unsigned long cas;
int zelena, cervena, modra;
void setupMalovani(void);
void loopMalovani(void);
void clearMalovani(void);
void barva(int a);
int x[pocetradku];

void test(void);

//timer
HardwareTimer timer(3);

//MyNeo
MyNeo LED = MyNeo();
myAdc ADC = myAdc();

// pototypy funkcí
void handler1(void);
void handler2(void);
void handler3(void);
void handler4(void);
void timerSetup(void);

void setup()
{
  for (int i = 0; i < 200; i++)
  {
    LED.setPixelColor(i, 0, 0, 0);
  }
  //debug
  Serial.begin(2000000);
  pinMode(PC14, OUTPUT);
  pinMode(PC15, OUTPUT);
  //multiplexer
  pinMode(PB11, OUTPUT);
  pinMode(PB12, OUTPUT);
  pinMode(PB13, OUTPUT);
  pinMode(PB14, OUTPUT);
  pinMode(PB15, OUTPUT);
  //IR LED a filtr
  pinMode(PA8, OUTPUT);
  pinMode(PB10, OUTPUT);
  timerSetup();
  setupMalovani();
}

void loop()
{

  if (LED.GetTC() == 1 && mereni == pocetmereni) //pokud je dokoncen prenos a mereni
  {
    if (kalibruj >= pocetkalibraci) //pokud je dokalibrovano
    {
      for (int j = 0; j < pocetradku; j++)
      {
        for (int i = 0; i < pocetsloupcu; i++)
        {
          rozdil[j][i] = values[j][i] - relative[j][i];
          if (ciltivost * pocetmereni < rozdil[j][i])
          {
            tlacitka[j][i] = 1;
            relative[j][i] += rozdil[j][i] / 100;
          }
          else if (-ciltivost > rozdil[j][i])
          {
            relative[j][i] = values[j][i];
            tlacitka[j][i] = 0;
          }
          else
          {
            tlacitka[j][i] = 0;
            relative[j][i] += rozdil[j][i] / 50;
          }
        }
      }
    }
    else
    {
      for (int j = 0; j < pocetradku; j++)
      {
        for (int i = 0; i < pocetsloupcu; i++)
        {
          relative[j][i] = values[j][i];
        }
      }
      kalibruj++;
    }
    loopMalovani();
    //test();
    LED.show();

    //odesli data
    radky++;
    if (radky >= pocetradku)
    {
      radky = 0;
    }
    Serial.println();
    Serial.print(radky);
    Serial.print("\t");
    for (sloupce = 0; sloupce < pocetsloupcu; sloupce++)
    {

      Serial.print(rozdil[radky][sloupce]);
      Serial.print("\t");
    }

    //zpracuj data
    char znaky = Serial.read();
    switch (znaky)
    {
    case 'c':
      for (int i = 0; i < 50; i++)
      {
        delay(100);
      }
      if (Serial.available() > 1)
      {
        ciltivost = Serial.parseInt();
      }
      break;
    default:
      Serial.read();
      break;
    }

    while (LED.GetTC() == 0) //ceká dokud neni dokoncen prenos do neopixel

      mereni = 0;
    timer.resume();
  }
}

void timerSetup(void)
{
  timer.pause();
  timer.setPrescaleFactor(72); //1MHz//      1us
  timer.setOverflow(300);
  timer.setChannel1Mode(TIMER_OUTPUT_COMPARE);
  timer.setCompare(TIMER_CH1, 0);
  timer.attachCompare1Interrupt(handler1);

  timer.setChannel2Mode(TIMER_OUTPUT_COMPARE);
  timer.setCompare(TIMER_CH2, 95);
  timer.attachCompare2Interrupt(handler2);

  timer.setChannel3Mode(TIMER_OUTPUT_COMPARE);
  timer.setCompare(TIMER_CH3, 130);
  timer.attachCompare3Interrupt(handler3);

  //aktualizuj casovac
  timer.refresh();
  // Start casovace
  timer.resume();
}

//  0 us
void handler1(void)
{
  gpio_write_bit(GPIOA, 8, HIGH); //nesviti
  gpio_write_bit(GPIOB, 10, LOW); //deriv
}

//  75 us
void handler2(void)
{
  gpio_write_bit(GPIOC, 14, LOW);             //debug
  if (mereni == 0 && row == 0 && group == 0)  //debud
  {
    gpio_write_bit(GPIOC, 15, LOW);           //debug
  }

  ADC.read();

  gpio_write_bit(GPIOC, 14, HIGH);            //debud
  gpio_write_bit(GPIOC, 15, HIGH);            //debug

  for (int i = 0; i < 10; i++)
  {
    if (mereni == 0) // vynuluj
    {
      values[row][i + group] = 0;
    }
    values[row][group + i] += ADC.values[i];
  }
  if (row == (pocetradku - 1) && group == 0)
  {
    row = 0;
    group = 10;
  }
  else if (row == (pocetradku - 1))
  {
    row = 0;
    group = 0;
    mereni++;
  }
  else
  {
    row++;
  }
}

//   100 us
void handler3(void)
{
  //prepne radek
  GPIOB->regs->BSRR = (0xf800 << 16) | ((mux[row + 10 - group] << 11) & 0xf800);
  //prepne radek bez sn74ls238 tzn. pouze 5 řádků
  //GPIOB->regs->BSRR = (0xf800 << 16) | (~(0x0800 << row) & 0xf800);
  gpio_write_bit(GPIOA, 8, LOW);   //sviti
  gpio_write_bit(GPIOB, 10, HIGH); //deriv
  if (mereni >= pocetmereni)
  {
    timer.pause();
    mereni = pocetmereni;
  }
}

void test()
{
  for (int i = 0; i < pocetradku; i++)
  { // cte radky
    for (int j = 0; j < pocetsloupcu; j++)
    { // cte sloupce
      if (tlacitka[i][j] == 1)
      {
        LED.setPixelColor((i * pocetsloupcu + j), 0, 0, 0);
      }
      else
      {
        LED.setPixelColor((i * pocetsloupcu + j), 20, 0, 0);
      }
    }
  }
}

//malovani
void setupMalovani()
{
  for (int i = 0; i < pocetradku; i++)
  {
    barva(i);
    LED.setPixelColor(((i * pocetsloupcu)), cervena, zelena, modra);
  }
}

void barva(int a)
{
  switch (a)
  {
  case 0:
  {
    zelena = 0;
    cervena = 0;
    modra = 0;
    break;
  }
  case 1:
  {
    zelena = 0;
    cervena = 255;
    modra = 0;
    break;
  }
  case 2:
  {
    zelena = 255;
    cervena = 0;
    modra = 0;
    break;
  }
  case 3:
  {
    zelena = 0;
    cervena = 0;
    modra = 255;
    break;
  }
  case 4:
  {
    zelena = 255;
    cervena = 255;
    modra = 0;
    break;
  }
  case 5:
  {
    zelena = 80;
    cervena = 255;
    modra = 0;
    break;
  }
  case 6:
  {
    zelena = 0;
    cervena = 255;
    modra = 20;
    break;
  }
  case 7:
  {
    zelena = 255;
    cervena = 255;
    modra = 255;
    break;
  }
  case 8:
  {
    zelena = 0;
    cervena = 128;
    modra = 128;
    break;
  }
  case 9:
  {
    zelena = 128;
    cervena = 0;
    modra = 128;
    break;
  }
  }
}

void loopMalovani()
{
  for (int i = 0; i < pocetradku; i++)
  { // cte radky
    for (int j = 1; j < pocetsloupcu; j++)
    { // cte sloupce
      if (tlacitka[i][j] == 1)
      {
        LED.setPixelColor((i * pocetsloupcu + j), cervena, zelena, modra);
      }
    }
  }
  for (int i = 0; i < pocetradku; i++)
  {
    if (tlacitka[i][0] == 1)
    {
      if (x[i] == 0)
      {
        cas = millis();
        x[i] = 1;
      }
      barva(i);
      if (millis() - cas > 2000)
      {
        clearMalovani();
      }
    }
    else
    {
      x[i] = 0;
    }
  }
}

void clearMalovani()
{
  for (int i = 0; i < pocetradku; i++)
  { // cte radky
    for (int j = 1; j < pocetsloupcu; j++)
    { // cte sloupce
      LED.setPixelColor((i * pocetsloupcu + j), cervena, zelena, modra);
    }
  }
}